@@ -3,10 +3,11 @@ from collections.abc import MutableMapping
 from jinja2 import Template
 from ploomber_core.exceptions import modify_exceptions
 import sql.connection
-from IPython.core.error import UsageError
 import warnings
 import difflib
 
+from sql import exceptions
+
 
 class SQLStore(MutableMapping):
     """Stores SQL scripts to render large queries with CTEs
@@ -43,15 +44,15 @@ class SQLStore(MutableMapping):
 
     def __getitem__(self, key) -> str:
         if not self._data:
-            raise UsageError("No saved SQL")
+            raise exceptions.UsageError("No saved SQL")
         if key not in self._data:
             matches = difflib.get_close_matches(key, self._data)
             error = f'"{key}" is not a valid snippet identifier.'
             if matches:
-                raise UsageError(error + f' Did you mean "{matches[0]}"?')
+                raise exceptions.UsageError(error + f' Did you mean "{matches[0]}"?')
             else:
                 valid = ", ".join(f'"{key}"' for key in self._data.keys())
-                raise UsageError(error + f" Valid identifiers are {valid}.")
+                raise exceptions.UsageError(error + f" Valid identifiers are {valid}.")
         return self._data[key]
 
     def __iter__(self) -> Iterator[str]:
@@ -71,12 +72,14 @@ class SQLStore(MutableMapping):
     @modify_exceptions
     def store(self, key, query, with_=None):
         if "-" in key:
-            raise UsageError(
-                "Using hyphens in save argument isn't allowed."
-                " Please use dashes(-) instead"
+            raise exceptions.UsageError(
+                "Using hyphens (-) in save argument isn't allowed."
+                " Please use underscores (_) instead"
             )
         if with_ and key in with_:
-            raise ValueError(f"Script name ({key!r}) cannot appear in with_ argument")
+            raise exceptions.UsageError(
+                f"Script name ({key!r}) cannot appear in with_ argument"
+            )
 
         self._data[key] = SQLQuery(self, query, with_)
 
@@ -100,18 +103,18 @@ class SQLQuery:
 
     def __str__(self) -> str:
         """
-        Since some dialects don't support " (double quote) symbol, we will
-        replace to the ' (backtick) symbol if it's supported
+        We use the ' (backtick symbol) to wrap the CTE alias if the dialect supports
+        ` (backtick)
         """
         with_clause_template = Template(
-            """WITH{% for name in with_ %} "{{name}}" AS ({{saved[name]._query}})\
+            """WITH{% for name in with_ %} {{name}} AS ({{saved[name]._query}})\
 {{ "," if not loop.last }}{% endfor %}{{query}}"""
         )
         with_clause_template_backtick = Template(
             """WITH{% for name in with_ %} `{{name}}` AS ({{saved[name]._query}})\
 {{ "," if not loop.last }}{% endfor %}{{query}}"""
         )
-        is_use_backtick = sql.connection.Connection.is_use_backtick_template()
+        is_use_backtick = sql.connection.Connection.current.is_use_backtick_template()
         with_all = _get_dependencies(self._store, self._with_)
         template = (
             with_clause_template_backtick if is_use_backtick else with_clause_template
